﻿using System;
using System.Security.Claims;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Tsi1.Api.Infrastructure;
using Tsi1.Api.Models;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.Api.Controllers
{
    [Authorize]
    [Route("api/[controller]")]
    [ApiController]
    public class UserController : ControllerBase
    {
        private readonly IJwtAuthManager _jwtAuthManager;
        private readonly IUserService _userService;
        private readonly IUserTypeService _userTypeService;

        public UserController(IJwtAuthManager jwtAuthManager, IUserService userService, IUserTypeService userTypeService)
        {
            _jwtAuthManager = jwtAuthManager;
            _userService = userService;
            _userTypeService = userTypeService;
        }

        [AllowAnonymous]
        [HttpPost("Login")]
        public async Task<IActionResult> Login(LoginRequest request)
        {
            var result = await _userService.Authenticate(request.UserName, request.Password);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            var user = result.Data;

            var claims = new[]
            {
                new Claim("Id", user.Id.ToString()),
                new Claim("Username", user.Username),
                new Claim(ClaimTypes.Role, user.UserType.Name)
            };

            var jwtResult = _jwtAuthManager.GenerateTokens(user.Username, claims, DateTime.Now);

            return Ok(new LoginResult
            {
                UserName = user.Username,
                Role = user.UserType.Name,
                AccessToken = jwtResult.AccessToken,
                RefreshToken = jwtResult.RefreshToken.TokenString
            });
        }

        [Authorize(Roles = UserTypes.FacultyAdmin)]
        [HttpPost("Register")]
        public async Task<IActionResult> Register(UserRegisterDto dto)
        {
            var userTypeResult = await _userTypeService.GetById(dto.UserTypeId);

            if (userTypeResult.HasError)
            {
                BadRequest(userTypeResult.Message);
            }

            var userType = userTypeResult.Data;

            var userServiceResult = await _userService.Create(dto, userType.Name);

            if (userServiceResult.HasError)
            {
                BadRequest(userServiceResult.Message);
            }

            return Ok();
        }
    }
}
